@@ -2,9 +2,9 @@
 
 //! pprof-rs is an integrated profiler for rust program.
 //!
-//! This crate provides a programable interface to start/stop/report a profiler dynamically. With the
-//! help of this crate, you can easily integrate a profiler into your rust program in a modern, convenient
-//! way.
+//! This crate provides a programable interface to start/stop/report a profiler
+//! dynamically. With the help of this crate, you can easily integrate a
+//! profiler into your rust program in a modern, convenient way.
 //!
 //! A sample usage is:
 //!
@@ -21,7 +21,22 @@
 //!};
 //! ```
 //!
-//! You can find more details in [README.md](https://github.com/tikv/pprof-rs/blob/master/README.md)
+//! More configuration can be passed through `ProfilerGuardBuilder`:
+//!
+//! ```rust
+//! let guard = pprof::ProfilerGuardBuilder::default().frequency(1000).blocklist(&["libc", "libgcc", "pthread"]).build().unwrap();
+//! ```
+//!
+//! The frequency means the sampler frequency, and the `blocklist` means the
+//! profiler will ignore the sample whose first frame is from library containing
+//! these strings.
+//!
+//! Skipping `libc`, `libgcc` and `libpthread` could be a solution to the
+//! possible deadlock inside the `_Unwind_Backtrace`, and keep the signal
+//! safety.
+//!
+//! You can find more details in
+//! [README.md](https://github.com/tikv/pprof-rs/blob/master/README.md)
 
 /// Define the MAX supported stack depth. TODO: make this variable mutable.
 pub const MAX_DEPTH: usize = 32;
@@ -39,7 +54,7 @@ mod timer;
 pub use self::collector::{Collector, HashCounter};
 pub use self::error::{Error, Result};
 pub use self::frames::{Frames, Symbol};
-pub use self::profiler::ProfilerGuard;
+pub use self::profiler::{ProfilerGuard, ProfilerGuardBuilder};
 pub use self::report::{Report, ReportBuilder};
 
 #[cfg(feature = "flamegraph")]
