@@ -7,6 +7,9 @@ use backtrace::Frame;
 use nix::sys::signal;
 use parking_lot::RwLock;
 
+#[cfg(any(target_arch = "x86_64", target_arch = "aarch64"))]
+use findshlibs::{Segment, SharedLibrary, TargetSharedLibrary};
+
 use crate::collector::Collector;
 use crate::error::{Error, Result};
 use crate::frames::UnresolvedFrames;
@@ -23,6 +26,92 @@ pub struct Profiler {
     sample_counter: i32,
 
     running: bool,
+
+    #[cfg(all(any(target_arch = "x86_64", target_arch = "aarch64")))]
+    blocklist_segments: Vec<(usize, usize)>,
+}
+
+pub struct ProfilerGuardBuilder {
+    frequency: c_int,
+    #[cfg(all(any(target_arch = "x86_64", target_arch = "aarch64")))]
+    blocklist_segments: Vec<(usize, usize)>,
+}
+
+impl Default for ProfilerGuardBuilder {
+    fn default() -> ProfilerGuardBuilder {
+        ProfilerGuardBuilder {
+            frequency: 99,
+
+            #[cfg(all(any(target_arch = "x86_64", target_arch = "aarch64")))]
+            blocklist_segments: Vec::new(),
+        }
+    }
+}
+
+impl ProfilerGuardBuilder {
+    pub fn frequency(self, frequency: c_int) -> Self {
+        Self { frequency, ..self }
+    }
+    #[cfg(all(any(target_arch = "x86_64", target_arch = "aarch64")))]
+    pub fn blocklist<T: AsRef<str>>(self, blocklist: &[T]) -> Self {
+        let blocklist_segments = {
+            let mut segments = Vec::new();
+            TargetSharedLibrary::each(|shlib| {
+                let in_blocklist = match shlib.name().to_str() {
+                    Some(name) => {
+                        let mut in_blocklist = false;
+                        for blocked_name in blocklist.iter() {
+                            if name.contains(blocked_name.as_ref()) {
+                                in_blocklist = true;
+                            }
+                        }
+
+                        in_blocklist
+                    }
+
+                    None => false,
+                };
+                if in_blocklist {
+                    for seg in shlib.segments() {
+                        let avam = seg.actual_virtual_memory_address(shlib);
+                        let start = avam.0;
+                        let end = start + seg.len();
+                        segments.push((start, end));
+                    }
+                }
+            });
+            segments
+        };
+
+        Self {
+            blocklist_segments,
+            ..self
+        }
+    }
+    pub fn build(self) -> Result<ProfilerGuard<'static>> {
+        trigger_lazy();
+
+        match PROFILER.write().as_mut() {
+            Err(err) => {
+                log::error!("Error in creating profiler: {}", err);
+                Err(Error::CreatingError)
+            }
+            Ok(profiler) => {
+                #[cfg(all(any(target_arch = "x86_64", target_arch = "aarch64")))]
+                {
+                    profiler.blocklist_segments = self.blocklist_segments;
+                }
+
+                match profiler.start() {
+                    Ok(()) => Ok(ProfilerGuard::<'static> {
+                        profiler: &PROFILER,
+                        timer: Some(Timer::new(self.frequency)),
+                    }),
+                    Err(err) => Err(err),
+                }
+            }
+        }
+    }
 }
 
 /// RAII structure used to stop profiling when dropped. It is the only interface to access profiler.
@@ -39,21 +128,7 @@ fn trigger_lazy() {
 impl ProfilerGuard<'_> {
     /// Start profiling with given sample frequency.
     pub fn new(frequency: c_int) -> Result<ProfilerGuard<'static>> {
-        trigger_lazy();
-
-        match PROFILER.write().as_mut() {
-            Err(err) => {
-                log::error!("Error in creating profiler: {}", err);
-                Err(Error::CreatingError)
-            }
-            Ok(profiler) => match profiler.start() {
-                Ok(()) => Ok(ProfilerGuard::<'static> {
-                    profiler: &PROFILER,
-                    timer: Some(Timer::new(frequency)),
-                }),
-                Err(err) => Err(err),
-            },
-        }
+        ProfilerGuardBuilder::default().frequency(frequency).build()
     }
 
     /// Generate a report
@@ -118,9 +193,53 @@ fn write_thread_name(current_thread: libc::pthread_t, name: &mut [libc::c_char])
 
 #[no_mangle]
 #[allow(clippy::uninit_assumed_init)]
-extern "C" fn perf_signal_handler(_signal: c_int) {
+#[cfg_attr(
+    not(all(any(target_arch = "x86_64", target_arch = "aarch64"))),
+    allow(unused_variables)
+)]
+extern "C" fn perf_signal_handler(
+    _signal: c_int,
+    _siginfo: *mut libc::siginfo_t,
+    ucontext: *mut libc::c_void,
+) {
     if let Some(mut guard) = PROFILER.try_write() {
         if let Ok(profiler) = guard.as_mut() {
+            #[cfg(all(any(target_arch = "x86_64", target_arch = "aarch64")))]
+            if !ucontext.is_null() {
+                let ucontext: *mut libc::ucontext_t = ucontext as *mut libc::ucontext_t;
+
+                #[cfg(all(target_arch = "x86_64", target_os = "linux"))]
+                let addr =
+                    unsafe { (*ucontext).uc_mcontext.gregs[libc::REG_RIP as usize] as usize };
+
+                #[cfg(all(target_arch = "x86_64", target_os = "macos"))]
+                let addr = unsafe {
+                    let mcontext = (*ucontext).uc_mcontext;
+                    if mcontext.is_null() {
+                        0
+                    } else {
+                        (*mcontext).__ss.__rip as usize
+                    }
+                };
+
+                #[cfg(all(target_arch = "aarch64", target_os = "linux"))]
+                let addr = unsafe { (*ucontext).uc_mcontext.pc as usize };
+
+                #[cfg(all(target_arch = "aarch64", target_os = "macos"))]
+                let addr = unsafe {
+                    let mcontext = (*ucontext).uc_mcontext;
+                    if mcontext.is_null() {
+                        0
+                    } else {
+                        (*mcontext).__ss.__pc as usize
+                    }
+                };
+
+                if profiler.is_blocklisted(addr) {
+                    return;
+                }
+            }
+
             let mut bt: [Frame; MAX_DEPTH] =
                 unsafe { std::mem::MaybeUninit::uninit().assume_init() };
             let mut index = 0;
@@ -155,8 +274,21 @@ impl Profiler {
             data: Collector::new()?,
             sample_counter: 0,
             running: false,
+
+            #[cfg(all(any(target_arch = "x86_64", target_arch = "aarch64")))]
+            blocklist_segments: Vec::new(),
         })
     }
+
+    #[cfg(all(any(target_arch = "x86_64", target_arch = "aarch64")))]
+    fn is_blocklisted(&self, addr: usize) -> bool {
+        for libs in &self.blocklist_segments {
+            if addr > libs.0 && addr < libs.1 {
+                return true;
+            }
+        }
+        false
+    }
 }
 
 impl Profiler {
@@ -193,8 +325,13 @@ impl Profiler {
     }
 
     fn register_signal_handler(&self) -> Result<()> {
-        let handler = signal::SigHandler::Handler(perf_signal_handler);
-        unsafe { signal::signal(signal::SIGPROF, handler) }?;
+        let handler = signal::SigHandler::SigAction(perf_signal_handler);
+        let sigaction = signal::SigAction::new(
+            handler,
+            signal::SaFlags::SA_SIGINFO,
+            signal::SigSet::empty(),
+        );
+        unsafe { signal::sigaction(signal::SIGPROF, &sigaction) }?;
 
         Ok(())
     }
@@ -216,17 +353,16 @@ impl Profiler {
 }
 
 #[cfg(test)]
+#[cfg(target_os = "linux")]
 mod tests {
     use super::*;
+
     use std::cell::RefCell;
     use std::ffi::c_void;
 
-    #[cfg(not(target_os = "linux"))]
-    #[allow(clippy::wrong_self_convention)]
-    static mut __malloc_hook: Option<extern "C" fn(size: usize) -> *mut c_void> = None;
+    use std::ptr::null_mut;
 
     extern "C" {
-        #[cfg(target_os = "linux")]
         static mut __malloc_hook: Option<extern "C" fn(size: usize) -> *mut c_void>;
 
         fn malloc(size: usize) -> *mut c_void;
@@ -293,6 +429,7 @@ mod tests {
         prime_numbers
     }
 
+    #[cfg(target_os = "linux")]
     #[test]
     fn malloc_free() {
         trigger_lazy();
@@ -307,7 +444,7 @@ mod tests {
         for i in 2..50000 {
             if is_prime_number(i, &prime_numbers) {
                 _v += 1;
-                perf_signal_handler(27);
+                perf_signal_handler(27, null_mut(), null_mut());
             }
         }
         unsafe {
